function SCS   =   AddDisco2SCS(SCS)  

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - AddDisco2SCS.m
%
% This function updates a Qualitative Representation so to enable the
% handling of discontinuities.
%
% Syntax:
%   SCS = AddDisco2SCS(SCS)
%
% Inputs:
%   SCS     :   structure describing the Shape Constrained Spline function.
%
% Outputs:
%   SCS     :   updated structure describing the Shape Constrained Spline
%               function. 
%
% -------------------------------------------------------------------------
% Latest version: Kris Villez, 2013-05-14
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

% this function adjusts the knots and QR for discontinuities

% setup parameter
% delta       =   1e-4            ;   % distance to multiple knot for approximation
% smax        =   3               ;   % number of sign levels

% input info
tk          =   SCS.tk     ;   % knot locations 
ty          =   SCS.ty     ;   
QR          =   SCS.QR     ;   % qualitative representations
m           =   size(QR,2)  ;   % number of episodes
order  =   SCS.order;
% signs       =   QR((1:smax)+2,:);   % sign requirements

t1          =   QR(1,:)     ;   % left episode bound
t2          =   QR(2,:)     ;   % right episode bound
disc        =   QR(6,:)     ;   % discontinuity (multiplicity)

if any(disc)
    
    % -------------------------------------------------------------------------
    % ADD KNOTS
    
    % find additional knots at discontinuity
    td                  =   [   ]   ;       % initialize
    for j=1:m-1                             % for each interior bound
        if disc(j)>=1                       % if discontinuity is present
            if t1(j+1)>t2(j)                % if location is not specified exactly
                % add the right and left interior bounds as well as all knots
                % in between as knots of multiplicity 4
                td_add  = [ t2(j)   ;   t1(j+1) ;   ty(and(ty>=t2(j),ty<=t1(j+1)))  ]   ;
%                 td_add  =   ones(3,1)*unique(td_add(:))'                                ;
                td_add  =   ones(disc(j),1)*unique(td_add(:))'                                ;
            elseif  t1(j+1)==t2(j)          % if the location of the discontinuity is specified exactly
                % add the specified interior bounds with required multiplicity
                td_add  =   t2(j)                               ;
                td_add  =   ones(disc(j),1)*unique(td_add(:))'  ;
            else
                error
            end
            td          =   [ td ; td_add(:) ]  ;   % add to vector
 
        end
    end
%     td
    
    % remove first and last knots: they cannot hold a discontinuity
    td          =   td(~ismember(td,[tk(1); tk(end)])) ;
%     td
    
    % add knots and update SCS
    if ~isempty(td)
        tk      =   setdiff(tk,td)              ;
        tk      =   sort([ tk(:) ; td(:)])      ;
                
%         figure, stairs(tk,'.-')
%         pause
        SCS    =   CreateSCS(SCS.y,SCS.ty,tk,order)          ;
    end
end

SCS.QR = QR;
return

% 
% 
% % -------------------------------------------------------------------------
% % UPDATE QR
%             
% % expand QR for discontinuities
% for j=m-1:-1:1          % run down over segments
%     if disc(j)>=1           % if discontinuity
%         
%         % insert slack interval so that signs are evaluated at the left and
%         % right
%         QR              =   [   QR(:,1:j)     QR(:,j:end)       ]   ;
%         t1(j+1)         =       min([   t2(end) t1(j+1)+delta   ])  ;
%         t2(j)           =       max([   t1(1)   t2(j)-delta     ])  ;
%         QR(2,j+(0:1))   =   [   t2(j)  t1(j+1)  ];
%         QR(1,j+(1:2))   =   [   t2(j)  t1(j+1)   ];
%         
%         smax                =   3               ; % number of sign levels
%         signvec             =   nan(smax,1)     ; % sign vector for slack interval
%         QR((1:smax)+2,j+1)  =   signvec         ; % slack interval has no sign specifications
%         QR(smax+1:end,j)    =   0               ; % set left episode as without discontinuity
%    
%     end
% end
% 
% % update SCS
% SCS.QR  =   QR  ;