function varargout = BoundsSCS(SCS,IntX,DistX,graph,verify)

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Boundlow.m
%
% This function executes the bounding procedures.
%
% Syntax:
%   [LB,UB,verified,X]   =   Bounds(SCS,IntX,DistX,graph,verify)
%
% Inputs:
%   SCS     :   Structure describing the Shape Constrained Spline function
%   IntX    :   Polyhedral set of transitions
%   DistX   :   Distance constraints for transitions
%   graph   :   boolean indicating whether graphs need to be produced
%   verify  :   boolean indicating whether shape verification is executed
%
% Outputs:
%   LB      :   lower bound objective value
%   UB      :   upper bound objective value
%   verified:   boolean indicating whether all transitions were located
%                   with absolute precision
%   X       :   best upper bound solution
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-06-18
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

% graph = 1;
% ===============================================
% I. Preparations

% -----------------------------------------------
% derive from SCS:
ty      =   SCS.ty    ;
DT      =   [ min(diff(ty)) ;   max(ty)-min(ty) ];
%QRs         =   SCS.QR    ;

nodisc = AnyDisc(SCS)     ;

% -----------------------------------------------
% derive from IntX:
nt      =   size(IntX,2) ;

% -----------------------------------------------
% settings
if nargin<3 || isempty(DistX),  DistX =   DT(:)*ones(1,nt-1) ;   end % default distance limits set to maximal range
if nargin<4 || isempty(graph),  graph   =   0               ;   end % no vizualization by default
if nargin<5 || isempty(verify), verify  =   0               ;   end % no shape verify by default

% ===============================================
% II. Actual computations

% -----------------------------------------------
% 1. FEASIBILITY

% try to find a feasible solution
[X,Feasible]    =   SCSfeasible(IntX,DistX,SCS)   ;

% Initialize empty solution
SolX = [];

% -----------------------------------------------
% 2. EVALUATE BOUNDS


if Feasible % if a feasible solution is available
    tk      =   SCS.tk                      ;

    time    =   tk(1):.01:tk(end) ; 
    % -----------------------------------------------
    % 2A. Evaluate lower bound
    [LB,OBJ_L]  =   BoundlowSCS(SCS,IntX)       ;
%     title('low')
%     disp(1)
    if isnan(LB)
        LB = 0 ;
        UB = inf;
        verified = false ;
    else
        if graph,   
            han=PlotSCS(OBJ_L,time); 
            h = get(han(1),'title');
            set(h,'String','Lower Bound solution');
        end

        if ~isfield(SCS,'UB') || LB<=SCS.UB
            % -----------------------------------------------
            % 2B. Evaluate upper bound
            [UB,OBJ_U,SolX]  =   BounduppSCS(SCS,IntX,X)         ;
            if isnan(UB)
                UB  =   inf ;
            else
                if graph,
                    han=PlotSCS(OBJ_U,time);
                    h = get(han(1),'title');
                    set(h,'String','Upper Bound solution');
                end
            end
        else
            %         disp('      Skipping computation upper bound: lower bound is higher than existing upper bound')
            SolX = [];
            UB = Inf ;
        end
% disp(2)
        % -----------------------------------------------
        % 2C. Shape verification and checking
        if nargout>=2
            
            verified    =   false               ; % default
            if isnan(LB)
                LB = 0 ;
            else
                if verify
                    verified    =   SCSverify_Case1(OBJ_L,IntX) ;
                    if verify>=2 && nodisc
                        verified    =   SCSverify_Case3(OBJ_L)  ;   %verify the shape of lower bound solution
                        if verified
%                             PlotSCS(OBJ_L)
%                             pause
                        end
                    end
                end
            end
            
            if verified             % if shape is verified
                UB  =   LB      ;   %   set upper bound equal to lower bound
            else
                DB  =   UB-LB   ;   %   check bound values for possible inconsistensies
                if DB<-inf ; %1e-6
                    %                 if abs(DB)<10^-3    %  very small difference
                    %                     LB  =   UB  ;   %   assign upper bound to lower bound
                    %                 else
                    %                     disp('Something not right: Upper bound is below lower bound')
                    %                     disp(TimeInt)       ;
                    %                     disp(Times)         ;
                    %                     disp([LB UB])       ;
                    %                     disp(DB)            ;
                    LB  =   0   ; % UB-abs(UB-LB) ;
                    UB  =   Inf ;
                    %                 end

                end
                UB  =   LB+abs(DB)  ;
            end
        end
    end
else
    % Not feasible: put bounds to +Inf
    LB          =   Inf     ;
    UB          =   Inf     ;
    verified    =   0       ;
end

if isnan(LB),   LB = Inf;   end
if isnan(UB),   UB = Inf;   end
% ===============================================
% III. Output
varargout{1}        =   LB          ;
if nargout>=2
    varargout{2}    =   UB          ;
    varargout{3}    =   verified    ;
    varargout{4}    =   SolX        ;
end

% [ verify verified LB UB ]

% ===============================================
return

