function [UBmin,SCSmin,SolXmin]  =   BounduppSCS(SCS,IntX,SolX)  

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Boundupp.m
%
% This function computes the upper bound SCS fit.
%
% Syntax:
%   [UBmin,SCSmin,SolXmin]  =   Boundupp(SCS,IntX,SolX)  
%
% Inputs:
%   SCS     :   Structure describing the Shape Constrained Spline function
%   IntX    :   Polyhedral set of transitions
%   SolX    :   Proposed solutions as row vectors
%
% Outputs:
%   UBmin   :   mimimum objective value among all proposed solutions
%   SCSmin  :   [optional] Shape Constrained Spline with best solution
%   SolXmin :   [optional] Best solution
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-06-18
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

[oo,PriorUB]   =   PriorBounds(IntX,SCS)    ;

tk      =   SCS.tk      ;   % knot locations
tk      =   tk(:)       ;   %   as column vector
% SolX =   Knots2Times(KnotSol,tk)       ;    

% initialized
UBmin   =   Inf             ;   % upper bound
SCSmin  =   SCS             ;   % generative trend objective
nSol    =   size(SolX,1) ;   % number of solutions to evaluate

% SolX

for iSol=1:nSol % for each solution
    
    X        =   SolX(iSol,:)     ;   % get current solution
    X        =   X(:)'            ;   %   as row vector
    if any(diff(X)<=0)   % check chronology
        %   invalid chronology: set upper bound to infinity
        UB = Inf;           
    else
        % valid chronology: evaluate QR fit
        % 1. update QR
        QRs      =   SCS.QR      ;   % qualitative representation
        V       =   length(QRs) ;
        if V> 1
            error('Unsupported functionality - Only one QR allowed')
        else

            v = 1;
            QR  =   QRs{v}  ;
            signs   =   QR(3:5,:)   ;
            disc    =   QR(6,:)     ;

            QR      =   [   tk(1)   X     ;
                            X       tk(end)   ;
                            signs           ;
                            disc            ];
%             QR
            QRs{v}  =   QR ;
        end
       
        SCS.QR  =   QRs             ;
        % 2. evaluate function fit
        SCS     =   SCS.FitFun(SCS)     ;
        % 3. obtain upper bound
        UB      =   SCS.J           ;
    end

    % upper bound is minimum of default and updated upper bound:
    UBmin       =   min([UB UBmin])       ;
    if UBmin==UB    % if computed upper bound is the new minimum
        SCSmin = SCS ; % update reported SCS
        SolXmin = X(iSol,:) ;
    end
end

UBmin                  =   UBmin + PriorUB                ;