function SCS = CreateSCS(y,ty,tk,order,lambda)

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - CreateSCS.m
%
% This function creates a structure describing a Shape Constrained Spline
% function.
%
% Syntax:
%   SCS = CreateSCS(y,ty,tk,order,lambda,options)
%
% Inputs:
%   y       :   measurements (column vector)
%   ty      :   [optional] measurement arguments (column vector)
%               [default]  uniformly spaced integers
%   tk      :   [optional] knot locations (column vector)
%               [default]  equal to ty
%   order	:   [optional] order of spline
%               [default]  4 (cubic spline)
%   lambda  :   [optional] regularization parameters for derivative 0 until
%               order of spline.  
%               [default] all zeros
%
% Outputs:
%   SCS     :   structure describing a Shape Constrained Spline function.
%
% -------------------------------------------------------------------------
% Latest version: Kris Villez, 2013-06-06
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

% =========================================================================
% DATA PRE-PROCESSING

[R,C]   =   size(y)      ;   % rows/ columns
if nargin<2 || isempty(ty)
    ty  =   1:R             ;   % generate equally-spaced dummy time vector
end
if nargin<3 || isempty(tk)
    tk  =   ty              ;   % if no knot vector is supplied, use time vector (results in interpolating or smoothing spline)
end
if nargin<4 || isempty(order)
    order  =   4*ones(1,C)             ;   % if no order vector is supplied, use cubic splines (order 4)
end
% default regularization penalty parameters
if nargin<5 || isempty(lambda)
    %lambda  =   [ zeros(1,3)  1 ]                ;
    lambda  =   [ zeros(1,3)  0 ]                ;
end
LossFun         =   @L2L2   ;

tk      =   tk(:)           ;   % as column vector
ty      =   ty(:)           ;   % as column vector
% y       =   y(:)            ;   % as column vector

inc     =   any(~isnan(y),2)       ;
ty      =   ty(inc,:)         ;
y       =   y(inc,:)          ;
% ny      =   length(y)       ;


% =========================================================================
% FUNCTIONAL DATA PART
nk      =   length(tk)          ;   % number of knots

trng    =   [tk(1), tk(end)]    ;   % range of knots


SCS.type       =   'SCS'      ;
SCS.order      =   order      ;
SCS.lambda     =   lambda      ;
SCS.tk         =   tk          ;
SCS.ty         =   ty          ;
SCS.y          =   y           ;


SCS.FitFun     =   @FitSCS ;
SCS.LossFun    =   LossFun ;

SCS.natural    =   1   ;
% % redmem  =   false ;

X2P     =   cell(1,1)   ;
L       =   cell(1,C)   ;
B       =   cell(1,C)   ;
Bk      =   cell(max(order),C)   ;
x       =   cell(1,C)   ;
bases   =   cell(1,C)   ;

for c=1:C
    order = order(c);
    
    % =========================================================================
    % get basis
    spline  =   'nspline'           ;   % type of spline
    if order<=2
        spline  =   'bspline'           ;   % type of spline
    end
    switch spline                       % conditional to spline
        case 'bspline'
            nx      =   nk + order - 2     ;   % number of basis functions
            basis   =   create_bspline_basis(trng, nx, order, tk)   ;  % evaluate basis
        case 'nspline'
            nx      =   nk + order - 4     ;   % number of basis functions
			basis   =   create_nspline_basis(trng, nx, order, tk)   ;  % evaluate basis
    end
    bases{c}    =   basis           ;
    x{c}        =   zeros(nx,1)     ;   % initialize coefficients
    %     nx
    % =========================================================================
    % basis evaluated in knots
    for d=0:order-1
        Bk{d+1,c}   =   eval_basis(tk,basis,d)    ;
    end
    B{c}   =   eval_basis(ty,basis,0)    ;
    
    % evaluate penalty matrices
    Pen     =   cell(order,C) ;
    for p=1:order
        switch p
            %                 case order
            %                     if causal
            %                         Pen1  =   X2U'*X2U ; %eye(nx)                 ;
            %                     else
            %                         Pen1  =    eye(nx)                 ;
            %                     end
            case order
                %                     dtk     =   diff(tk)                ;
                %                     Bk3     =   Bk{end} ;
                %                     IB3k    =   diag(dtk)*Bk3(1:nk-1,:) ;
                Pen1  =  speye(nx) ; % IB3k'*IB3k              ;
            otherwise
                Pen1  =   eval_penalty(basis,p-1)   ;
        end
        %             Pen1        =   speye(nx)   ;
        Pen{p,c}    =   Pen1        ;
    end
    
    
    % ---------------------
    % For SOCP:
    n           =   order       ;
    for d   =   0:n-1
        X2P{n-d,c} =   GetMapsX2P(tk,Bk,d,n);
    end
    
    
    L{c}       =   0*speye(nx)       ;
    for j=1:order
        L{c}   =   L{c} + lambda(j) * Pen{j,c}  ;
    end
    %         J(c)       =   LossFun(B{c},L{c},x{c},y(:,c))  ;
    
end

SCS.J  =   nan ; %sum(J)                   ;
SCS.x      =   x       ;
SCS.x0     =   x       ;
SCS.X2P 	=   X2P     ;
SCS.L      =   L       ;

% % =========================================================================
% % STORAGE
SCS.bases      =   bases   ;
SCS.x          =   x       ;
SCS.y          =   y       ;
SCS.nk         =   nk      ;
SCS.lambda     =   lambda  ;
SCS.Pen        =   Pen     ;

% % basis evaluated in data locations
SCS.ty     =   ty      ;
SCS.B0y    =   B       ;

SCS.modeltype  =   'SCS'   ;


SCS = Presolve(SCS) ;

return

