
% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - Demo.m
%
% This script demonstrates the use the SCS and SPIKE_O toolboxes for
% generalized Shape Constrained Spline function fitting. 
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-06-26
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

clc 
clear all
close all

UsingMosek = true ; % set to false to use CVX instead

% =========================================================================

disp('0. Load and plot data:')
load titanium_data

plottime = x(1):.01:x(end);

figure, plot(x,y,'ko')

disp(' paused')
pause

% =========================================================================

disp('1. Create SCS object and fit unconstrained spline:')
tk      =       x(1:1:end)          ;
tx      =   [   tk(1):1:tk(end) ]   ;
tk      =   [   tk([1:1:end])     ]   ;

lambda      =   [   0 0  10^(-7)  0 ]   ;
order       =   4 ; 
SCSO        =   CreateSCS(y(:),x(:),tk(:),order,lambda)       ;

if UsingMosek
    SCSO.optimizer	=	'mosek' ;
else
    SCSO.optimizer	=	'cvx'   ;
end

SCS1        =   FitSCS(SCSO)        ;

PlotSCS(SCS1,plottime);

disp(' 	Notice the fluctuations in the 1st derivative of the fitted function')
disp(' paused')
pause

% =========================================================================

disp('2. Add shape constraint as in Turlach et al. (2005) and fit Shape Constrained Spline accordingly:') 

SCS2    =       SCSO ;
QR      =   [   595 835 955     ;   % episode start
                835 955 1075    ;   % episode end
                nan(1,3)        ;   % sign for function value
                nan(1,3)        ;   % sign for 1st derivative
                1   nan  1      ;   % sign for 2nd derivative
                zeros(1,3)    ] ;   % 0/1 indicator for discontinuity (currently not supported, all zero)

disp('	Applied QR: ')            
disp(['      Episode start: ' num2str(QR(1,:),'%5.0f') ])
disp(['      Episode end:   ' num2str(QR(2,:),'%5.0f') ])            
disp(['      Primitive:     ' ' P    Q    P'] )            

SCS2.QR{1}  =   QR              ;

SCS2        =   FitSCS(SCS2)    ;
PlotSCS(SCS2,plottime);

disp(' 	Notice how the fluctuations in the 1st derivative of the fitted function have disappeared')
disp(' paused')
pause

% =========================================================================

disp('3. Compute bounds:') 

QR      =   [   595 886 955     ;   % episode start
                835 910 1075    ;   % episode end
                nan(1,3)        ;   % sign for function value
                1 nan -1        ;   % sign for 1st derivative
                1   -1  1       ;   % sign for 2nd derivative
                zeros(1,3)    ] ;   % 0/1 indicator for discontinuity (currently not supported, all zero)
         
SCS3        =   SCSO    ;
SCS3.QR{1}  =   QR      ;

graph   =   1   ;
verify  =   0   ;
IntX    =   tk([ 9 25 ; 17  33]); 
DistX   =   []  ;

disp('	Applied intervals for transitions: ')            
disp([ '      Transition 1: ' num2str(IntX(:,1)','%5.0f')])    
disp([ '      Transition 2: ' num2str(IntX(:,2)','%5.0f')])

[FL,FU,ver,T] = BoundsSCS(SCS3,IntX,DistX,graph,verify);

disp('	Computed solution for upper bound: ')            
disp([ '      Transition 1: ' num2str(T(:,1)','%5.2f')])    
disp([ '      Transition 2: ' num2str(T(:,2)','%5.2f')])
disp('')
disp('	Applied QR for upper bound: ')            
disp(['      Episode start: ' num2str([QR(1,1) T],'%5.0f') ])
disp(['      Episode end:   ' num2str([T QR(2,end)],'%5.0f') ])            
disp(['      Primitive:     ' ' B    N    A'] )        
disp('') 
disp('	Applied QR for lower bound: ')            
disp(['      Episode start: ' num2str([QR(1,1) IntX(2,:)],'%5.0f') ])
disp(['      Episode end:   ' num2str([IntX(1,:) QR(2,end) ],'%5.0f') ])            
disp(['      Primitive:     ' ' B    N    A'] )        

disp('	Computed bounds: ')            
disp([ '      Lower bound: ' num2str(FL,'%5.6f')])    
disp([ '      Upper bound: ' num2str(FU,'%5.6f')])

disp(' paused')
pause

% =========================================================================
% Branch-and-Bound
disp('4. Branch-and-bound search:') 

SCS4        =   SCSO    ;
SCS4.QR{1}  =   QR      ;

nk          =   SCSO.nk             ;
nt          =   size(SCS4.QR{1},2)-1   ;
IntX        =   tk([1 nk]'*ones(1,2))  ;
DistX       =   []                  ;   

disp('	Applied root set for transitions: ')            
disp([ '      Transition 1: ' num2str(IntX(:,1)','%5.0f')])    
disp([ '      Transition 2: ' num2str(IntX(:,2)','%5.0f')])

options         =   OptimsetBB(SCS4);
options.resolution  =   10               ;
options.verify  =   0               ;

disp('')
disp('	Colour code: ')    
disp('      Blue    = fathomed node')    
disp('      Pink    = verified solution (upper bound = lower bound)')    
disp('      Black   = selected solution')    
disp('  Please take note of above colour codes to understand subsequent vizualization of the branch-and-bound algorithm')    
disp(' paused')
pause

[Tree,SCS4]    =   BB(SCS4,options,IntX,DistX)    ;

[QR4,trans] = SCS2QR(SCS4);
 
disp('')
disp('	Obtained optimal solution: ')            
disp([ '      Transition 1: ' num2str(trans(1)','%5.3f')])    
disp([ '      Transition 2: ' num2str(trans(2)','%5.3f')])

disp(' paused')
pause

figure(99)
    FigPrep
    AxisPrep
    set(gca,'Xtick',[plottime(1):60:plottime(end)])
    set(gca,'Ytick',[plottime(1):60:plottime(end)])
    if exist('xticklabel_rotate')
        % if interested,get this function at:
        %   http://www.mathworks.com/matlabcentral/fileexchange/3486-xticklabelrotate
        hText = xticklabel_rotate;
    end
    xlabel('Temperature [^oC]','Position',[.5 -0.1]);
    ylabel('Temperature [^oC]');

PlotSCS(SCS4,plottime);    

PlotQRT(QR4,3)

disp('	Plotted optimal solution with graphical representation')            
disp(' paused')
pause

% =========================================================================
% Display all fits
y1 = EvalSCS(SCS1,plottime);
y2 = EvalSCS(SCS2,plottime);
y4 = EvalSCS(SCS4,plottime);
dy1 = EvalSCS(SCS1,plottime,1);
dy2 = EvalSCS(SCS2,plottime,1);
dy4 = EvalSCS(SCS4,plottime,1);
figure
FigPrep
    LW =2;
subplot(2,1,1)
    AxisPrep
    plot(x,y,'ko')
    plot(plottime,y1,'k-','linewidth',LW)
    plot(plottime,y2,'k-','linewidth',LW)
    plot(plottime,y4,'k-','linewidth',LW)
    legend({'data','spline fits'},2)
    set(gca,'Xlim',[plottime(1) plottime(end)],'Xticklabel',[])
    ylabel('y')
subplot(2,1,2)
    AxisPrep
    plot(plottime,dy1,'k:','linewidth',LW)
    plot(plottime,dy2,'k--','linewidth',LW)
    plot(plottime,dy4,'k-','linewidth',LW)
    d = 0.002;
    set(gca,'ylim',[-d d])
    set(gca,'Xlim',[plottime(1) plottime(end)])
    xlabel('Temperature [^oC]')
    ylabel('dy/dt')
    legend({'Q','PQP','BNA'},'Location','Best');
        

disp('	Plotted curves (top) and first derivatives (bottom)')   

return
