function SCS = FitSCS(SCS)

% -------------------------------------------------------------------------
% SCS toolbox - FitSCS.m
%
% Fit shape constrained spline function
% 
% Syntax is as follows:
%   SCS = FitSCS(SCS)
%
% Inputs: 	
%   SCS :   Structure defining Shape Constrained Spline
% Outputs: 	
%   SCS :   Updated structure defining Shape Constrained Spline
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-05-20
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

verbose =   0   ;


if verbose, disp('  Get data'), end

X       =   SCS.x      ;
Y       =   SCS.y      ;

V       =   size(Y,2)     ;


% -------------------------------------------------------------------------
% if or(isfield(SCS,'QR'),~strcmp(FunStr,'L2L2'))
    
if verbose, disp('  Input checking'), end
    
if isfield(SCS,'QR')
    QRs     =   SCS.QR         ;   % qualitative representation
    if ~iscell(QRs)
        QRs =   { QRs } ;
    end
else
    QR      =   [  SCS.tk(1) SCS.tk(end) nan(1,3) 0 ]';
    QRs     =   cell(V,1)  ;
    [QRs{:}]=   deal(QR) ;
    SCS.QR =   QRs             ;
end

JJ = nan(1,V) ;

for v=1:V
    
    y   =   Y(:,v)  ;
    
    SCSv        =   SCS            ;
    SCSv.x      =   X(v)            ;
    SCSv.y      =   y               ;
    SCSv.order =   SCS.order(v)  ;
    
    SCSv.B0y    =   SCS.B0y(v)     ;
    SCSv.bases  =   SCS.bases(v)   ;
    SCSv.basis  =   SCSv.bases{1}   ;
    SCSv.X2P    =   SCS.X2P(:,v)   ;
    SCSv.L      =   SCS.L(v)       ;
    
    QR  =   QRs{v}  ;
    
    C   =   size(QR,2)      ;   % number of primitives
    if C>0
        tt  =   QR(1:2,:)       ;   % episode endpoints
        dt  =   tt(2,:)-tt(1,:) ;   % episode lengths
        
        QR(3:5,dt<0)    =   nan         ;   % put signs to nan if episode bounds are not chronological
        QR(1,dt<0)      =   QR(2,dt<0)  ;   % set right endpoints equal to left endpoints
        tt              =   QR(1:2,:)   ;   % updated episode endpoints
    end
    
    if verbose, disp('  Input processing'), end
    % -------------------------------------------------------------------------
    % Obtain reduced, non-verlapping intervals for episodes
    if C>0
        ttred           =	tt      ;   % initialize
        for c=1:C
            % left episode should be smaller than all bounds to the left
            ttred(1,c)  = 	min(tt(1,c),max([ tt(1,c) tt(1,1:c-1) tt(2,1:c-1) ]) )	;
            % right episode should be larger than all bounds to the right
            ttred(2,c)  = 	max(tt(2,c),min([ tt(2,c) tt(1,c:end) tt(2,c:end) ]) )	;
        end
        tt          =   ttred   ;   % update endpoints
        QR(1:2,:)   =   tt      ;   % update QR
    end
    
    SCSv.QR     =   QR      ;   % update object
    
    if C>0
        SCSv    =   AddDisco2SCS(SCSv)  ;   % add knots for specified discontinuity
        QR      =   SCSv.QR             ;   % get updated QR
        
    end
    tk      =   SCSv.tk             ;   % get update knot arguments
    
    SCS.bases(v)   =   SCSv.bases(1)   ;
    SCS.B0y(v)     =   SCSv.B0y        ;
    
    % nx      =   length(SCS.x)      ;   % number of coefficients
    % ny      =   length(SCS.y)      ;   % number of observations
    
    % -------------------------------------------------------------------------
    % Expand QR: if gaps are present in QR, add episodes in between to make all episodes
    % contiguous
    if C>0
        QR      =   ExpandQR(QR)        ;
    end
    
    % -------------------------------------------------------------------------
    % Generate interval specifications
    if verbose, disp('  Generate interval specifications'), end
    
    if C>0
        QR      =   QR'                             ;   % transpose
        inc     =   find(and(QR(:,2)>=tk(1),QR(:,1)<=tk(end)))   ;
        QR      =   QR(inc,:)                       ;
        QR(:,1) =   max(QR(:,1),tk(1))              ;
        QR(:,2) =   min(QR(:,2),tk(end))            ;
        
    end
    QRs{v} =   QR' ;
    
    if verbose, disp('  Second Order Cone Program (SOCP)'), end
    
    [x,J,Fit,Pen]       =   SolveSOCP(SCSv,verbose)  ;  % solve SOCP
    
    SCSv.x      =   x               ;
    Yhat(:,v)   =   EvalSCS(SCSv)   ;
    X{v}        =   x               ;
    JJ(v)       =   J               ;
    
    %     figure, hold on, plot(y), plot(x,'.')
end

% SCS = SCSv ;
if verbose, disp('  Store solution'), end
SCS.x      =   X       ;   % coefficients
% sum(JJ)
SCS.J      =   sum(JJ)       ;   % objective function value
SCS.Jfit   =   Fit     ;
SCS.Jpen   =   Pen     ;
SCS.QR     =   QRs     ;
SCS.Yhat   =   Yhat    ;
