function [SolX,feasible] = SCSfeasible(IntX,DistX,SCS)    

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - SCSfeasible.m
%
% This function executes shape verification.
%
% Syntax:
%   [SolX,feasible] = SCSfeasible(IntX,DistX,SCS)    
%
% Inputs:
%   IntX    :   Interval specification for feasible set
%   DistX   :   Distance specification for feasible set
%   SCS     :   Structure describing the Shape Constrained Spline function
%
% Outputs:
%   SolX    :   Proposed solution
%                   be found with absolute precision 
%   feasible:   Boolean vector indicating whether a feasible solution has
%                   been found
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-06-18
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

% -----------------------------------------------
% Derive from SCS:
tk  =   SCS.tk     ;
nk  =   length(tk)  ;
kk  =   1:nk            ;
T   =   [ 1 nk  ]       ;
% -----------------------------------------------
% Derive from IntX:
nib     =   size(IntX,2) ;   % number of interior bounds
maxd    =   T(2)-T(1)       ;   % maximum distance

if nargin<2 || isempty(DistX),   DistX    =   [1 maxd]'*ones(1,nib-1)  ;   end % default distance

% choose strategy to find optimal strategy. 
%   1: QP (quadratic program)
%   2: DP (dynamic program)
strat   =   1               ;

switch strat
    case 1  % quadratic program
        
%         Feasible point is solution to a QP objective function: sum of
%         squared distances
        % ------------------------------------------------
%         get weights for distance metric
%         QR      =   SCS.QR{1}     ;   % qualitative representation
%         signs   =   QR(3:5,:)   ;   % derivative signs
% 
% %         Weights are generally all set to one. When zeros are present,
% %         those are included only (set to one)
%         w           =   zeros(nib+1,1)      ;   % initialize to zero
%         for j=1:nib+1
%             w(j)    =   any(signs(:,j)==0)  ;   % if zero-valued, set to one
%         end
%         w           =   ~w                  ;   % negative of given values
        
        % ------------------------------------------------
        
        % Ut    : upper limit for transition location, relative to knot locations
        % Lt    : lower limit for transition location, relative to knot locations
        % Ud    : upper limit for transition location distance, relative to knot locations
        % Ld    : lower limit for transition location distance, relative to knot locations

        Ut  =   IntX(2,:)    ;
        Lt  =   IntX(1,:)    ;
        Ud  =   DistX(2,:)   ;
        Ld  =   DistX(1,:)   ;
        
        Ut  =   Ut(:)  ;
        Lt  =   Lt(:)  ;
        Ud  =   Ud(:)  ;
        Ld  =   Ld(:)  ;
        
        
        if isfield(SCS,'optimizer')
            optimizer = SCS.optimizer;
        else
            optimizer = 'mosek';
        end
        e = ones(nib-1,1);
        A = [   spdiags([+e -e ], 0:1, nib-1, nib); ...
            spdiags([-e +e ], 0:1, nib-1, nib)];
        b = [ -Ld(:); Ud(:) ];

        switch optimizer
            case 'cvx'
                cvx_begin
                    cvx_quiet true
                    variables t(nib,1)
                    minimize ( pow_pos(norm(t-Lt),2)+pow_pos(norm(t-Ut),2))
                    subject to
                        A*t   <=  b ;
                        t    >=  Lt  ;
                        t    <=  Ut  ;
                cvx_end
                t   =   t(:)' ;
                feasible    =  strcmp(cvx_status,'Solved') ;
                
            case 'mosek'
                % t^2 - 2* t*Lt' + Lt^2 + t^2 - 2* t*Ut' + Ut^2
                % = 2*t^2 - 2* t*(Lt+Ut)' + Lt^2 + Ut^2
                % ~= .5 *  (2 * t^2) - 1.t*(Lt+Ut)' 
				
				
                % (t - Lt).^2 + (t-Ut).^2  + (t(2..) -t(1..)).^2 				+ (t(1)-Lt(1)).^2  						+(t(end)-Ut(end)).^2
				%  							( [ -1 1 .. ;  0 -1 1 ..].t )^2   	+ 1 t(1)^2 - 2. Lt(1).t(1) + Lt(1)^2  	+ 1 t(end)^2 - 2. Ut(end).t(end) + Ut(end)^2 
				
                H   =   2*eye(nib)          ;
                f   =   -(Lt+Ut)            ;
				% adjust edges
                t   =   quadprog(H,f,A,b,[],[],Lt,Ut) ;
                t   =   t(:)' ;
                feasible = ~isempty(t) ;
        end
        
%         if feasible
%             plot(t(1),t(2),'k.')
%         end
                
%                 t
        if feasible
            SolX   =   t   ;
        else
            SolX   =   []  ;
        end
        
    case 2
		% solve as a dynamic program
        % ------------------------------------------------
        % get indicators for equality constraints
        QRs         =   SCS.QR         ;   % qualitative representation
        QRmap       =   SCS.QR         ;
        V           =   length(QRs)     ;
        for v=1:V
            QR      =   QRs{v} ;
            signs   =   QR(3:5,:)       ;   % derivative signs
            disc    =   QR(6,:)         ;   % derivative signs
        
            % w indicates zero-valued sign
            w           =   zeros(1,nib)    ;
            for j=1:nib
                if ismember(j,QRmap{v})
                    w(j,v)    =   and(any([signs(:,j); signs(:,j+1)]==0),disc(j)==0)  ;
                else
                    w(j,v) = 0;
                end
            end
        end
        w   =   any(w,2)    ;
        
        % ------------------------------------------------
        % evaluate possibility for solution
        feasible    =   1                                   ; % initialize feasibility
        %X           =   unique(sort([kk(:) ; IntX(:) ])) ;   % values
        X           =   unique(sort([tk(:) ; IntX(:) ])) ;   % values
        cc          =   2:nib                               ;
%         cc          =   1:nib                               ;
        anyzero     =   w(cc)                               ;
        KnotInt     =   Times2Knots(IntX,tk)             ;
        atresol     =   (KnotInt(2,cc)-KnotInt(1,cc))<=1    ;
        check       =   and(anyzero(:),atresol(:))                ;
%         check
        if any(check)
            cc      =   cc(check)       ;
            check   =   check(check)    ;
            nc      =   length(cc)      ;
            for ic=1:nc
                if ~all(KnotInt(:,cc(ic)-1)==KnotInt(:,cc(ic)))
                    check(ic)   =   0   ;
                end
            end
            % update feasibility: if any check remains true -> infeasible
            feasible = ~any(check)  ;
            
        end
        % ------------------------------------------------
        % if still assumed feasible, try to find a solution
        if feasible 
            
            IntX         =   IntX'        ;   % rotate matrix
            K               =   cell(nib+2,1)   ;
            K{1}            =   1               ;   % set initial node value (dummy)
            for it=1:nib % for each interior bound
                if w(it) % zero-valued section
                    % find available knot locations
                    K{it+1} =   find(and(and(X>=IntX(it,1),X<=IntX(it,2)),ismember(X,tk)))'   ;
                else
                    % find available locations (knot and non-knot)
                    K{it+1} =   find(and(X>=IntX(it,1),X<=IntX(it,2)))'   ;
                end
            end
            K{nib+2}            =   length(X)               ;   % add final node (dummy)
%             DistX
            L                   =   [ 0 Inf ; DistX']    ;   % add interval bound for first distance (to first dummy node)
            [feasible,SolX]    =   DynProg(K,L,X)          ;   % execute dynamic program
            SolX               =   SolX(2:end-1)'         ;   % remove first and last point
        else
            SolX               =   nan(nib,1)  ;
        end
end

