function [verified,trans] = SCSverify_Case2(SCS)

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - SCSverify_Case2.m
%
% This function executes shape verification.
%
% Syntax:
%   [verified,trans] = SCSverify_Case3(SCS)
%
% Inputs:
%   SCS         :   structure describing the Shape Constrained Spline function.
%
% Outputs:
%   verified    :   boolean indicating whether or not all transitions could
%                   be found with absolute precision 
%   trans       :   [optional] verified transition locations (nan for those
%                   not found)
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-05-20
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

graph   =   0           ;   % vizualization (0/1)

QRs     =   SCS.QR      ;   % qualitative representation

V   =   length(QRs) ;
% if V>1
%     QRmap   =   SCS.QRmap   ;
% else
%     QRmap{1}    =   
% end


ListVerified    =   nan(V,1)    ;
% ListTrans       =   cell(V,1) 	;

for v=1:V
    QR  =   QRs{v}  ;
    nep     =   size(QR,2)  ;   % number of episodes
    if nep==1 % only one episode
        verified    =   true        ;
        trans       =   [   ]       ;
    else

%         pause
        Sr      =   QR(3:5,:)   ;   % required signs
        trans   =   QR(1:2,:)   ;   % episode bounds

        % ----------------------------------------------------------
        % check for overlap
        overlap             =   zeros(3,nep)                        ;   % initialize overlap parameter
        for r=2:nep-1   % for each interior episode
            overlap(1,r)    =   any(trans(2,1:r-1) > trans(1,r))    ;   % check if any previous episode overlaps with this one
            overlap(2,r)    =   any(trans(1,r+1:end) < trans(2,r))  ;   % check if any later episode overlaps with this one
        end
        overlap(3,:)        =   trans(1,:)>=trans(2,:)              ;   % check if any episode is ill-defined with left bound beyond right bound
        overlap             =   any(overlap(:))                     ;   % if any value is true then overlap is true

        % ----------------------------------------------------------
        % actual checking
        if overlap % no verification is impossible in the case of overlap
            verified    =   false           ;
            trans       =   nan(nep-1,1)    ;
        else
            % verify solution if no overlap
            x       =   SCS.x       ;   % coefficients
            trans   =   trans(:)    ;   % all episode bounds as column vector
            
            tc      =   sort(unique( [ SCS.tk(:) ; trans ]))    ;   % add episode bounds as evaluation points
            
%             SCS     =   BasisUpdate(SCS,tc)                     ;   % update basis
%             B0c     =   SCS.B0c ;   % basis for function evaluation
%             B1c     =   SCS.B1c ;   % basis for 1st derivative
%             B2c     =   SCS.B2c ;   % basis for 2nd derivative
% 
%             y0      =   B0c*x   ;   % function evaluation
%             y1      =   B1c*x   ;   % 1st derivative evaluation
%             y2      =   B2c*x   ;   % 2nd derivative evaluation

            y0  =   EvalSCS(SCS,tc,0)   ;
            y1  =   EvalSCS(SCS,tc,1)   ;
            y2  =   EvalSCS(SCS,tc,2)   ;
            
            crit    =   0                       ;   % critical value
            h0      =   sign(softcut(y0,crit))  ;   % sign after soft tresholding
            h1      =   sign(softcut(y1,crit))  ;   % sign after soft tresholding
            h2      =   sign(softcut(y2,crit))  ;   % sign after soft tresholding

            s0      =   sign(y0)    ;   % sign after (no tresholding)
            s1      =   sign(y1)    ;   % sign after (no tresholding)
            s2      =   sign(y2)    ;   % sign after (no tresholding)

            % for vizualization
            if graph

                % tx is usually defined over finer grid, therefore better for
                % vizualization
                tx      =   SCS.ty(:)   ;

                y0x     =   EvalSCS(SCS,tx,0)   ;
                y1x     =   EvalSCS(SCS,tx,1)   ;
                y2x     =   EvalSCS(SCS,tx,2)   ;

            end

            Se      =   [ s0(:)';  s1(:)'; s2(:)' ];    % combined evaluated signs
            He      =   [ h0(:)';  h1(:)'; h2(:)' ];    % combined evaluated signs (soft-tresholded)

            match   =   zeros(nep-1,1)  ;   % initialize matching of evaluated and required sign
            trans   =   nan(nep-1,1)    ;   % initialize identified episode bounds
    
            for q=1:nep-1 % for each interior bound
                t1  =   QR(2,q)     ;   % right episode bound of current episode
                t2  =   QR(1,q+1)   ;   % left episode bound of next episode

                if t1==t2   % if episode bounds are equal
                    match(q)    =   1   ;   % matching is true (no ambiguity)
                    trans(q)    =   t1  ;   % identified transition is at interior bound
                else

                    % check which sign changes (+1 to -1 or vice versa)
                    level   =   find(Sr(:,q)==-Sr(:,q+1)) ;

                    inc     =   and(tc>=t1,tc<=t2)  ;   % include evaluation point between two interior bounds
                    te      =   tc(inc)             ;   % included evaluation points
                    y0e     =   y0(inc)             ;   % function value
                    y1e     =   y1(inc)             ;   % 1st derivative
                    y2e     =   y2(inc)             ;   % 2nd derivative
                    tt      =   te                  ;   % evaluation points

                    if graph
                        figure
                        hold on
                        plot(tc,y1,'.')
                        plot(te,y1e,'o')
                        plot(tx,y1x{1},'-')
                    end

                    if isempty(level)   % no sign change (a.k.a. some zero is present)

                        zero1       =   find(Sr(:,q)==0,1,'first')  ;   % locate lowest level with zero in left episode
                        zero2       =   find(Sr(:,q+1)==0,1,'first');   % locate lowest level with zero in right episode

                        level       =   min([zero1 zero2])  ;   % locate lowest level with zero

                        %assert(~isempty(level),'Unresolved case: no effective transition')

                        s           =   He(level,inc)       ;   % find evaluated soft-tresholded sign
                        [tt,s]      =   shorten(tt,s)       ;   % reduce the sequence of signs to essential sequence

                        if length(tt)==2 && all(s==Sr(level,q:q+1))
                            % if length is 2 (meaning 1 evaluated transition) and
                            % obtained signs match requirements
                            loc     =   find(te==tt(2))     ;   % locate identified transition time
                            ts      =   te(loc-1:loc)       ;   % get transition time and time just before
                            %                         y0s     =   y0e(loc-1:loc)      ;   % get function value
                            %                         y1s     =   y1e(loc-1:loc)      ;   % get 1st derivative
                            %                         y2s     =   y2e(loc-1:loc)      ;   % get 2nd derivative
                            if s(1)==0                          % if left episode has zero
                                tt  =   ts(1)   ;               %   select left side for further checking
                            elseif s(2)==0                      % if right episode has zero
                                tt  =   ts(2)   ;               %   select right side for further checking
                            else
                                disp('error')
                            end

                            checks      =   SubCheck(level,q,tc,tt,t1,t2,Sr,He) ;   % check signs at other levels
                            trans(q)    =   tt                                  ;   % store identified transition time
                            match(q)    =   all(checks(:))                      ;   % evaluate whether all signs match
                        end
                    else
                        % verify whether there is a unique transition + locate transition

                        s       =   Se(level,inc)           ;   % find evaluated sign
                        [tt,s]  =   shorten(tt,s)           ;   % reduce the sequence of signs to essential sequence
                        if length(tt)==2 && all(s==Sr(level,q:q+1))
                            % if length is 2 (meaning 1 evaluated transition) and
                            % obtained signs match requirements
                            loc     =   find(te==tt(2))     ;   % locate identified transition time
                            ts      =   te(loc-1:loc)       ;   % get transition time and time just before
                            y0s     =   y0e(loc-1:loc)      ;   % get function value
                            y1s     =   y1e(loc-1:loc)      ;   % get 1st derivative
                            y2s     =   y2e(loc-1:loc)      ;   % get 2nd derivative
                            dt      =   ts(2)-ts(1)         ;   % time interval
                            
                            switch level
                                case 1 % zero order transition: crossing
                                    C   =   [ (y2s(2)-y2s(1))/(6*dt)  y2s(1)/2 y1s(1)  y0s(1) ]  ; % coefficients of cubic problem
                                    R   =   roots(C)            ;   % find roots
                                    R   =   R(and(R>=0,R<=dt))  ;   % select roots inside interval
                                    tt  =   ts(1)+R             ;   % absolute time of transition
                                case 2 % first order transition: extremum
                                    % find transition point
                                    C   =   [ (y2s(2)-y2s(1))/(2*dt)  y2s(1)  y1s(1) ]  ; % coefficients of quadratic problem
                                    R   =   roots(C)            ;   % find roots
                                    R   =   R(and(R>=0,R<=dt))  ;   % select roots inside interval
                                    tt  =   ts(1)+R             ;   % absolute time of transition
                                    %                          tt
                                    %                          figure
                                    %                             hold on
                                    %                             plot(tx,y1x,'.-')
                                    %                             plot(ts,y1s,'o')
                                    %                             plot(tt,0,'ro')
                                    %                          pause
                                case 3 % second order transition: inflection point
                                    tt  =   ts(1)+(ts(2)-ts(1))* (-y2s(1))/(y2s(2)-y2e(1)); % interpolate
                            end

                            checks      =   SubCheck(level,q,tc,tt,t1,t2,Sr,Se) ;   % check signs at other levels
                            trans(q)    =   tt                                  ;   % store identified transition time
                            match(q)    =   all(checks(:))                      ;   % evaluate whether all signs match
                        else
                            match(q)    =   0       ; % no match
                        end
                    end

                end
            end
            verified    =   all(match)  ;   % all interior bounds should be matched for a verified solution
        end
    end
    
    ListVerified(v)     =   verified    ;
%     ListTrans{v} =   trans(1:end-1)      ;
    
end

verified = all(ListVerified)    ;
% verified
% if verified
%     disp('verified solution')
% end

% -------------------------------------------------------------------------
function [tt,s] =   shorten(tt,s)       
    
for c=length(s)-1:-1:1 % go backward through sequence
    if s(c)==s(c+1) % if sign is the same in current and next instance: do reduce
        tt  = [ tt(1:c) ;   tt(c+2:end) ]   ;   % remove next instance
        s   = [ s(1:c)      s(c+2:end)  ]   ;   % remove next instance
    end
end

% -------------------------------------------------------------------------
function checks =   SubCheck(level,j,tc,tt,t1,t2,Sr,Se)

R               =   3           ;   % number of rows (levels)
C               =   2           ;   % number of columns (episodes to check)

checks          =   zeros(R,C)  ;   % initialize all checks
checks(level,:) =   1           ;   % set checks at level of change to one (verified already)

% verify remainder of QR
for c=1:C   % for each episode
    switch c    % condition on episode
        case 1, inc =   and(tc>=t1,tc<tt)   ;   % include evaluation points before transition
        case 2, inc =   and(tc>tt,tc<=t2)   ;   % include evaluation points after transition
    end
    for r   =   setdiff(1:R,level)              % for each level, except the one verified already
        if isnan(Sr(r,j+c-1))                   %   if sign of episode is nan
            checks(r,c)     =   1           ;   %       automatically assign true (matched)
        else                                    %   else
            s               =   Se(r,inc)   ;   %       evaluated sign
            if or(s==Sr(r,j+c-1),s==0)          %       if sign is equal to requirement or if is zero
                checks(r,c) =   1           ;   %           assign true (matched)
            end
        end
    end
end