function [x,J,Fit,Pen] =   SolveSOCP(SCS,verbose) 

% -------------------------------------------------------------------------
% Shape Constrained Splines toolbox - SolveSOCP.m
%
% This function defines and solves the Shape Constrained Spline fitting
% problem as a Second Order Cone Program (SOCP).
%
% Syntax:
%   [x,J,Fit,Pen] =   SolveSOCP(SCS,verbose) 
%
% Inputs:
%   SCS     :   structure describing a Shape Constrained Spline function.
%   verbose :   boolean indicating whether function should behave verbose.
%
% Outputs:
%   x       :   spline coefficients
%   J       :   objective function
%   Fit     :   lack-of-fit term in objective function
%   Pen     :   regularization term in objective function
%
% -------------------------------------------------------------------------
% Latest version: Kris Villez, 2013-05-14
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SCS Toolbox for Matlab/Octave. 
% 
% The SCS Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by the
% Free Software Foundation, either version 3 of the License, or (at your
% option) any later version.
% 
% The SCS Toolbox is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SCS Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------

if nargin <3 || isempty(verbose), verbose = false; end

% =========================================================================
% I.
% INITIAL SETUP
if verbose, disp('  ... Setup'), end

order   =   SCS.order          ;

% -----------
% ensure QR is viable for given SCS order:
QR      =   SCS.QR             ;
QR(order+3:end-1,:) =   nan     ;
SCS.QR =   QR                  ;
int     =   GetIntervals(SCS)  ;
% -----------

x       =   SCS.x{1}           ;  % current coefficients
y       =   SCS.y              ;  % observations

nx      =   length(x)           ;  % number of coefficients (x)
ny      =   length(y)           ;  % number of observations (y)

n       =   SCS.order          ;  % spline order

cc      =   [   1:2 6:8	]       ;   % include these columns always
INT = cell(order-1,1) ;
for j=0:order-1
    INT{j+1}  =   int(:,[ cc 3+j ]) ;   % constraint specifications for function constraints
end

Lossfun     =   SCS.LossFun    ;   % chosen loss function

[Bineq,Beq] =   GetPointConstraints(SCS)   ;
B0y         =   SCS.B0y{1}        ;  % spline basis evaluated in observation arguments
L           =   SCS.L{1}          ;  % regularization matrix

% =========================================================================
% II.
% FIND (1) MATRICES TO MAP SPLINE COEFFICIENTS TO POLYNOMIAL COEFFICIENTS
% FOR FUNCTION AND ALL RELEVANT DERIVATIVES AND (2) COEFFICIENTS TO DEFINE
% SECOND ORDER CONE CONSTRAINTS
if verbose, disp('  ... Generating constraint parameters'), end

% Initialize number of piece-wise inequalities/equalities
m0 = 0 ;    m0eq = 0 ;
m1 = 0 ;    m1eq = 0 ; 
m2 = 0 ;    m2eq = 0 ; 
m3 = 0 ;    m3eq = 0 ;
% Get the matrices
for d=n-4+(1:3)
    e   =   n-(d+1) ;
    switch d
        case 0
            % piece-wise polynomial representation of function
            % interval endpoints, coefficient to polynomial map, number of
            % inequality constraints, number of equality constraints
            [a0,b0,Map0,m0,m0eq]    =   GetMaps(SCS,INT{e+1},e,n)   ;
        case 1
            % piece-wise polynomial representation of 1st derivative
            [a1,b1,Map1,m1,m1eq]    =   GetMaps(SCS,INT{e+1},e,n)   ;
        case 2
            % piece-wise polynomial representation of 2nd derivative
            [a2,b2,Map2,m2,m2eq]    =   GetMaps(SCS,INT{e+1},e,n)   ;
        case 3
            % piece-wise polynomial representation of 2nd derivative
            [a3,b3,Map3,m3,m3eq]    =   GetMaps(SCS,INT{e+1},e,n)   ;
    end
end

if verbose, disp('  ... Solving'), end

x0      =   SCS.x      ;

if isfield(SCS,'optimizer')
    optimizer = SCS.optimizer;
else
    optimizer = 'mosek';
end

% [ m0 m1 m2 m3]

% =========================================================================
% III.
% FORMULATE LINEAR CONSTRAINS
if verbose, disp('  ... Formulating linear constraints'), end

% -------------------------------------------------------------------------
% 3.1 INTERVAL FUNCTION/DERIVATIVE INEQUALITIES 
if verbose, disp('      ... Interval inequality constraints'), end
if m0>=1
    % original formulation (unchanged)
    % derivative 3: polynomial coefficients p0
    %             Map0{1,1}*x ==  x0_0    ; % 2.a   p0
    %             x0_0    >=      0       ; % SOC-1

    %               x               x0_0        
    A0e     =   [   Map0{1,1}       -speye(m0)        ]   ;
    A0i     =   [   sparse(m0,nx)    +speye(m0)        ]   ;  
    
    A0      =   [   A0e ;...
                    A0i     ]       ;
    bb0e    =   sparse(m0,1)   ;
    bb0L    =   sparse(m0,1)   ;
    bb0U    =   inf(m0,1)   ;
    
end

if m1>=1
    % original formulation (unchanged):
    % derivative 2: polynomial coefficients p0..p1
    %             Map1{2,1}*x ==  -a1.*x1_0   +b1.*y1_0    ; % 2.a   p0
    %             Map1{1,1}*x ==       x1_0   -    y1_0    ; % 2.b   p1
    
    %               x               x1_0        y1_0
    A1e     =   [   Map1{2,1}       -diag(-a1)  -diag(+b1)      ;...
                    Map1{1,1}       -speye(m1)    speye(m1)     ]   ;
    A1i     =   [   sparse(m1*2,nx)  speye(m1*2)               ]   ;  
    
    A1      =   [   A1e ;...
                    A1i     ]       ;
    bb1e    =   sparse(m1*2,1)   ;
    bb1L    =   sparse(m1*2,1)   ;
    bb1U    =   inf(m1*2,1)   ;
    
end
                
if m2>=1
    % original formulation with intermediate variables (capitalized):
    %             % derivative 1: polynomial coefficients p0..p2
    %             Map2{3,1}*x	==      x2_0                    - a2.*b2  .*  y2_0    ;   % p0
    %             Map2{2,1}*x   ==              2*x2_1          + (a2+b2) .*  y2_0    ;   % p1
    %             Map2{1,1}*x	==                      x2_2    -             y2_0    ;   % p2
    %             X2_0    =       x2_0+x2_2                   ;
    %             X2_1    =   [   2*x2_1  ]   ;
    %             X2_2    =   [   x2_0-x2_2    ]   ;
    %             X2_0      >=      norms([X2_1 X2_2],2,2)              ; SOC-1
    %             Y2_0      >=      0              ;              SOC-2
    
    % 			Converted formulation: eliminate uncapitalized variables in
    % 			above and rename capitalized variables as uncapitalized versions:
    %             Map2{3,1}*x	==      x2_0/2          +x2_2/2 - a2.*b2  .*  y2_0    ;   % p0
    %             Map2{2,1}*x   ==              x2_1            + (a2+b2) .*  y2_0    ;   % p1
    %             Map2{1,1}*x	==      x2_0/2          -x2_2/2    -             y2_0    ;   % p2
    %             x2_0      >=      norms([x2_1 x2_2],2,2)              ; SOC-1
    %             y2_0      >=      0              ;              SOC-2

    %               x           x2_0        x2_1        x2_2        y2_0
    A2e     =   [   Map2{3,1}	-eye(m2)/2  zeros(m2)   -eye(m2)/2  -diag(-a2.*b2)      ;...    % p0
                    Map2{2,1}   zeros(m2)   -eye(m2)    zeros(m2)   -diag(a2+b2)        ;...    % p1
                    Map2{1,1}	-eye(m2)/2  zeros(m2)   +eye(m2)/2  +eye(m2)            ]    ;  % p2
%     A2e     = zeros(A2e) ;
    A2i     =   [   zeros(m2,nx+3*m2)                               eye(m2) ];
    
    
    A2      =   [   A2e ;...
                    A2i     ]       ;
    bb2e    =       zeros(m2*3,1)   ;
    bb2L    =       zeros(m2*1,1)   ;
    bb2U    =       inf(m2*1,1)     ;
end
           
if m3>=1
            % function sign: polynomial coefficients p0..p3
			% original formulation with intermediate variables (capitalized):

%             Map3{4,1}*x ==    ( -a3.*x3_0 + b3.*y3_0 )                      ;   % p0
%             Map3{3,1}*x ==  2*( -a3.*x3_1 + b3.*y3_1 )  +   ( x3_0 - y3_0 ) ;   % p1
%             Map3{2,1}*x ==    ( -a3.*x3_2 + b3.*y3_2 )	+2* ( x3_1 - y3_1 ) ;   % p2
%             Map3{1,1}*x ==                                  ( x3_2 - y3_2 ) ;   % p3

%			  X3_0      =       x3_0 + x3_2                 ;
%             X3_1     =   [     2*x3_1  ]   ;
%             X3_2      =   [   x3_0 - x3_2    ]   ;
%             X3_0      >=      norms([X3_1 X3_2],2,2)              ;   % SOC-1
%             Y3_0      =       y3_0 + y3_2                 ;
%             Y3_1     =   [    2*y3_1  ]   ;
%             Y3_2     =   [   y3_0 - y3_2     ]   ;
%             Y3_0      >=      norms([Y3_1 Y3_2],2,2)              ;   % SOC-1

% 			Converted formulation: eliminate uncapitalized variables in above and rename capitalized variables as uncapitalized versions:
%             Map3{4,1}*x ==    ( -a3.*(x3_0+x3_2)/2 + b3.*(y3_0+y3_2)/2 )                      ;   % p0
%             Map3{3,1}*x ==  2*( -a3.*x3_1/2 + b3.*y3_1/2 )  +   ( (x3_0+x3_2)/2 - (y3_0+y3_2)/2 ) ;   % p1
%             Map3{2,1}*x ==    ( -a3.*(x3_0-x3_2)/2 + b3.*(y3_0-y3_2)/2 )	+2* ( x3_1/2 - y3_1/2 ) ;   % p2
%             Map3{1,1}*x ==                                        ((x3_0-x3_2)/2 - (y3_0-y3_2)/2  ) ;   % p3
%             x3_0      >=      norms([x3_1 x3_2],2,2)              ;   % SOC-1
%             y3_0      >=      norms([y3_1 y3_2],2,2)              ;   %
%             SOC-1

    %               x           x3_0        x3_1        x3_2        y3_0        y3_1        y3_2
    A3e     =   [   Map3{4,1}   diag(a3/2)  zeros(m3)   diag(a3/2)  -diag(b3/2) zeros(m3)   -diag(b3/2)       ;...    % p0
                    Map3{3,1}   -eye(m3)/2    diag(a3)  -eye(m3)/2  +eye(m3)/2     -diag(b3) +eye(m3)/2   ;...    % p1
                    Map3{2,1}	diag(a3/2)  -eye(m3)  -diag(a3/2) -diag(b3/2)   eye(m3)    diag(b3/2)      ;...  % p2
                    Map3{1,1}	-eye(m3)/2   zeros(m3)    +eye(m3)/2   +eye(m3)/2   zeros(m3)    -eye(m3)      ]    ;  % p3
    A3e = sparse(A3e) ;
%     A2i     =   [   sparse(m2,nx+3*m2)                               speye(m2) ];
    
    
    A3      =   	A3e             ;
    bb3e    =       sparse(m3*4,1)   ;
end

% -------------------------------------------------------------------------
% 3.2 INTERVAL FUNCTION/DERIVATIVE EQUALITIES
if verbose, disp('      ... Interval equality constraints'), end
if m0eq>=1
    % Original cvx formulation:
    % derivative 1
    %     Map2{3,2}*x	==  sparse(m2eq,1)   ;   % p0
    %     Map2{2,2}*x ==  sparse(m2eq,1)   ;   % p1
    %     Map2{1,2}*x	==  sparse(m2eq,1)   ;   % p2
    
    E0      =   [   Map1{1,2}   ];          % p0
    e0      =       sparse(m0eq*2,1) ;
end

if m1eq>=1
    % Original cvx formulation:
    % derivative 1
    %     Map2{3,2}*x	==  sparse(m2eq,1)   ;   % p0
    %     Map2{2,2}*x ==  sparse(m2eq,1)   ;   % p1
    %     Map2{1,2}*x	==  sparse(m2eq,1)   ;   % p2
    
    E1      =   [   Map1{2,2}   ;...        % p0
                    Map1{1,2}   ];          % p1
    e1      =       sparse(m1eq*2,1) ;
end

if m2eq>=1
    % Original cvx formulation:
    % derivative 1
    %     Map2{3,2}*x	==  sparse(m2eq,1)   ;   % p0
    %     Map2{2,2}*x ==  sparse(m2eq,1)   ;   % p1
    %     Map2{1,2}*x	==  sparse(m2eq,1)   ;   % p2
    
    E2      =   [   Map2{3,2}   ;...        % p0
                    Map2{2,2}   ;...        % p1
                    Map2{1,2}   ];          % p2
    e2      =       sparse(m2eq*3,1) ;
end

if m3eq>=1
    % Original cvx formulation:
    %   function value
    %             Map3{4,2}*x ==  sparse(m3eq,1)   ;   % p0
    %             Map3{3,2}*x ==  sparse(m3eq,1)   ;   % p1
    %             Map3{2,2}*x ==  sparse(m3eq,1)   ;   % p2
    %             Map3{1,2}*x ==  sparse(m3eq,1)   ;   % p3

    E3      =   [   Map3{4,2}   ;...        % p0
                    Map3{3,2}   ;...        % p1
                    Map3{2,2}   ;...        % p2
                    Map3{1,2}   ];          % p3
    e3      =       sparse(m3eq*4,1) ;
end

% -------------------------------------------------------------------------
% 3.3 POINT-WISE CONSTRAINTS
if verbose, disp('      ... Point-wise constraints'), end
Ap = Bineq  ;    
Ep = Beq    ;   
ePe= sparse(size(Ep,1),1) ;
ePL = sparse(size(Ap,1),1) ;
ePU = inf(size(Ap,1),1) ;

% =========================================================================
% IV.
% SOLVE PROBLEM
if verbose, disp('  ... Solving problem'), end

    switch optimizer
            
        case {1,'mosek'}
            %         [m0 m1 m2 m3]
            SolveSOCPmosek
            
        case {2,'cvx'}
            % CVX based solution
            SolveSOCPcvx
            
    end

% =========================================================================
% IV.
% SOLVE PROBLEM
if verbose, disp('  ... Solved, storing solution'), end
[J,Fit,Pen] =   Lossfun(x,SCS)   ;
SCS = SCS ;


return


function int =   GetIntervals(SCS)         

QR  =   SCS.QR'  ;
tk  =   SCS.tk  ;

tt  =   QR(:,1:2)                       ;   % endpoints
tt  =   max(min(tt,tk(end)),tk(1))  ;
tab =   sort(unique([tk(:) ; tt(:)]))   ;   % constraint interval endpoint series

m   =   length(tk)      ;   % number of knots
nc  =   length(tab)-1   ;   % number of constraint intervals
int =   nan(nc,8)       ;   % initialize interval specifications

int(:,1)	=   tab(1:nc)       ;   % set left interval endpoint
int(:,2)    =   tab(2:nc+1)     ;   % set right interval endpoint

ne  =   size(QR,1)  ;   % number of episodes
for e=1:ne              % for each episode
%     if or(isnan(QR(e,4)),isnan(QR(e,5)))
        inc             =   find(and(int(:,1)>=QR(e,1),int(:,2)<=QR(e,2)))  ;   % included interval endpoint
        int(inc,3:5)    =   ones(length(inc),1)*QR(e,3:5)                   ;   % add sign specifications
%     end
end

for k=1:m-1 % for each knot interval
    % find the knot interval to which the constraint interval is associated
    inc             =   and(int(:,1)>=tk(k),int(:,2)<=tk(k+1) )   ;
    int(inc,6)      =   k       ;   % assign knot interval
end
int_k       =   int(:,6)    ;   % all knot interval specifications
int(:,7)    = ( int(:,1)-tk(int_k) ) ./ ( tk(int_k+1)-tk(int_k) ) ; % left interval endpoint relative to knot interval
int(:,8)    = ( int(:,2)-tk(int_k) ) ./ ( tk(int_k+1)-tk(int_k) ) ; % right interval endpoint relative to knot interval

nans    =   isnan(int(:,3:5))   ;
allnans =   all(nans,2)         ;
int     =   int(~allnans,:)     ;